import pymysql
from dbutils.pooled_db import PooledDB
import constants

"""
数据库连接池
    creator=pymysql,                         # 使用链接数据库的模块,这里使用的是pymysq模块
    maxconnections=10,                       # 连接池允许的最大连接数,这里设置的10,具体自己定义
    mincached=2,                             # 初始化时，链接池中至少创建的空闲的链接，0表示不创建
    maxcached=5,                             # 链接池中最多闲置的链接，0和None不限制
    maxshared=1,                             # 链接池中最多共享的链接数量
    blocking=True,                           # 连接池中如果没有可用连接后，是否阻塞等待。True，等待；False，不等待然后报错
    maxusage=None,                           # 一个链接最多被重复使用的次数，None表示无限制
    setsession=[],                           # 开始会话前执行的命令列表。
    host=constants.ETL_HOST,                 #数据库ip
    port=3306,                               #数据库端口
    user=constants.ETL_USER,                 #数据库用户名
    password=constants.ETL_PASSWORD,         #数据库密码
    database=constants.ETL_DBNAME,           #数据库库名
    charset='utf8'                           #数据库编码

"""
POOL = PooledDB(
    creator=pymysql,
    maxconnections=10,
    mincached=2,
    maxcached=5,
    maxshared=1,
    blocking=True,
    maxusage=None,
    setsession=[],
    host=constants.ETL_HOST,
    port=3306,
    user=constants.ETL_USER,
    password=constants.ETL_PASSWORD,
    database=constants.ETL_DBNAME,
    charset='utf8'
)


def create_conn():
    """

    :return: 返回连接和游标
    """
    conn = POOL.connection()
    cursor = conn.cursor(pymysql.cursors.DictCursor)
    return conn, cursor


def close_conn(conn, cursor):
    """

    :param conn:   连接
    :param cursor: 游标
    :return:
    """

    conn.close()
    cursor.close()
