import glob
import os
import shutil
import zipfile
import sys
import pandas as pd
import numpy as np
import re

sys.path.append('/root/dmp/etl_process_data/etl_job')

import modifyStatus, constants, DbUtils

import time

localTime = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
print(localTime)


def unzip_all(source_dir, dest_dir):
    """

    :param source_dir: 源目录
    :param dest_dir:   目标目录
    :return:
    """
    try:
        it = os.scandir(source_dir)
        for entry in it:
            if entry.is_file() and os.path.splitext(entry.name)[1] == '.zip':
                if not os.path.exists(dest_dir):
                    # os.mkdir(dest_dir)
                    print("file not exists")
                    # zipfile.ZipFile(entry.path).extractall(path=dest_dir)
                else:
                    zipfile.ZipFile(entry.path).extractall(path=dest_dir)
    except Exception as e:
        #   update file_status set PRE_FILE_PROCESSING,update file_code set UNZIP_CODE
        modifyStatus.updateStatusById(constants.PRE_FILE_PROCESSING, constants.UNZIP_CODE, e, localTime,
                                      constants.SP_UPDATE_PERSON, file_id,constants.PRE_FILE_TABLE_NAME)
        print("unzip_all error:{0}".format(e))
    return


def copy_file(srcPath, dtPath):
    try:
        for root, dirs, files in os.walk(srcPath, True):
            for fileName in files:
                if fileName.endswith('.csv') or fileName.endswith('.txt'):
                    shutil.copy(os.path.join(root, fileName), dtPath)
    except Exception as e:
        #   update file_status set PRE_FILE_PROCESSING,update file_code set FTP_CODE
        modifyStatus.updateStatusById(constants.PRE_FILE_PROCESSING, constants.FTP_CODE, e, localTime,
                                      constants.ZMH_UPDATE_PERSON, file_id,constants.PRE_FILE_TABLE_NAME)
        print("copy_file error:{0}".format(e))
    return 1


def analysis_csv(file_id, csv_path):
    """

    :param file_id: 文件id
    :param csv_path: csv路径
    :return:
    """
    csvFiles = glob.glob(csv_path + "/*")
    list_ = []
    list_error_file = []
    for files in csvFiles:
        try:
            df = pd.read_csv(files, index_col=None, header=0)
            list_.append(df)
        except Exception as e:
            #   update file_status set PRE_FILE_PROCESSING,update file_code set PARSE_CSV_FILE
            modifyStatus.updateStatusById(constants.PRE_FILE_PROCESSING, constants.PARSE_CSV_FILE, e, localTime,
                                          constants.ZMH_UPDATE_PERSON, file_id,constants.PRE_FILE_TABLE_NAME)
            list_error_file.append(files)
            continue
    data = pd.concat(list_, ignore_index=True)
    print(data)
    ten_json = data.head(2).to_json(orient='index')
    update_sql = "update t_file_pretreatment_status set explore_json='" + ten_json + "' where file_deposit=" + str(file_id)

    # trim
    try:
        stripstr = lambda x: x.strip() if isinstance(x, np.unicode) else x
        data = data.applymap(stripstr)
        for i in data.keys():
            col = str(data[i][0])
            compile1 = re.compile(
                '\d{4}[-/.]\d{1,2}[-/.]\d{1,2}\s\d{1,2}:\d{1,2}:\d{1,2}|\d{4}[-/.]\d{1,2}[-/.]\d{1,2}')
            match_all = compile1.findall(col)
            if len(match_all) != 0:
                if len(col) == 19:
                    data[i] = pd.to_datetime(data[i])
                elif len(col) == 10:
                    data[i] = pd.to_datetime(data[i])
        str_col = ""
        for i in range(0, len(data.dtypes)):
            #  example:(id,'name','int','2012-12-02 00:00:00','sp'),(id,'name','int','2012-12-02 00:00:00','sp'),
            str_col += "(" + str(file_id) + ", '" + str(data.columns.values[i]) + "', '" + str(data.dtypes[i]) + \
                       "', '" + str(localTime) + "', '" + str(constants.ZMH_UPDATE_PERSON) + "'),"
        # example:insert into table (id,name,age) value((1,'zh','se','2012-12-02 00:00:00'),(1,'sg','er','2012-12-02 00:00:00'))
        insert_sql = "insert into t_file_save_field_info (file_deposit,field_name,field_type,cre_time,cre_person) values " + str_col[0:-1]
        return list_error_file, update_sql, insert_sql
    except Exception as e:
        #   update file_status set PRE_FILE_PROCESSING,update file_code set TIME_FORMAT_CODE
        modifyStatus.updateStatusById(constants.PRE_FILE_PROCESSING, constants.CSV_TYPES_PARSE, e, localTime,
                                      constants.ZMH_UPDATE_PERSON, file_id,constants.PRE_FILE_TABLE_NAME)
        print("time format error{}".format(e))


if __name__ == '__main__':
    get_file_deposit_info = sys.argv
    print(get_file_deposit_info)
    file_id = get_file_deposit_info[1]
    get_ftp_path = get_file_deposit_info[3]
    get_output_hdfs_path = get_file_deposit_info[5]
    get_unzip_output_path = get_file_deposit_info[6]
    get_copy_output_path = get_file_deposit_info[7]
    #  update file_status set FILE_WAIT_PROCESS,update file_code set DEFAULT_PRETREATMENT_STATUS_RESULT
    modifyStatus.updateStatusById(constants.PRE_FILE_PROCESSING, constants.DEFAULT_FILE_STATUS,
                                  constants.DEFAULT_ERROR_INFO, localTime, constants.SP_UPDATE_PERSON,
                                  file_id,constants.PRE_FILE_TABLE_NAME)
    unzip_all(get_ftp_path, get_unzip_output_path)
    su_code = copy_file(get_unzip_output_path, get_copy_output_path)
    if su_code == 1:
        os.system("hdfs dfs -rm -r /user/tuomin_test")
        os.system("hdfs dfs -mkdir /user/tuomin_test")
    pre_list = analysis_csv(file_id, get_copy_output_path)
    print("csv无法读取的文件list：" + str(pre_list[0]))
    print("十条json数据更新sql：" + str(pre_list[1]))
    print("数据字段写入sql：" + str(pre_list[2]))
    db, cursor = DbUtils.create_conn()
    cursor = db.cursor()
    cursor.execute(pre_list[1])
    cursor.execute(pre_list[2])
    db.commit()
    DbUtils.close_conn(db, cursor)
    #   update file_status set FILE_PROCESSING_FINISH,update file_code set DEAL_SUCCESS
    modifyStatus.updateStatusById(constants.FILE_PROCESSING_FINISH, constants.FINAL_RESULT_SUCCESS, "成功", localTime,
                                  constants.SP_UPDATE_PERSON, file_id,constants.PRE_FILE_TABLE_NAME)
    # except Exception as e:
    #     print("主函数update执行异常：{}".format(e))
    #     modifyStatus.updateStatusById(constants.PRE_FILE_PROCESSING, constants.MAIN_UPDATE_FILE, e, localTime,
    #                                   constants.ZMH_UPDATE_PERSON, file_id,constants.PRE_FILE_TABLE_NAME)